@@ -1,10 +1,7 @@
-# -*- coding: utf-8 -*-
 """
 Google Cloud Storage pythonic interface
 """
 import asyncio
-import fsspec
-
 import io
 import json
 import logging
@@ -13,23 +10,27 @@ import posixpath
 import re
 import warnings
 import weakref
+from datetime import datetime, timedelta, timezone
+from urllib.parse import parse_qs
+from urllib.parse import quote as quote_urllib
+from urllib.parse import urlsplit
 
-from fsspec.asyn import sync_wrapper, sync, AsyncFileSystem
-from fsspec.utils import stringify_path, setup_logging
+import fsspec
+from fsspec.asyn import AsyncFileSystem, sync, sync_wrapper
 from fsspec.callbacks import NoOpCallback
 from fsspec.implementations.http import get_client
-from .retry import retry_request, validate_response
+from fsspec.utils import setup_logging, stringify_path
+
+from . import __version__ as version
 from .checkers import get_consistency_checker
 from .credentials import GoogleCredentials
-from . import __version__ as version
-from urllib.parse import quote as quote_urllib
-from urllib.parse import parse_qs, urlsplit
+from .retry import retry_request, validate_response
 
 logger = logging.getLogger("gcsfs")
 
 
 if "GCSFS_DEBUG" in os.environ:
-    setup_logging(logger=logger, level=os.environ["GCSFS_DEBUG"])
+    setup_logging(logger=logger, level=os.getenv("GCSFS_DEBUG"))
 
 
 # client created 2018-01-16
@@ -48,7 +49,7 @@ bACLs = {
     "publicRead",
     "publicReadWrite",
 }
-DEFAULT_PROJECT = os.environ.get("GCSFS_DEFAULT_PROJECT", "")
+DEFAULT_PROJECT = os.getenv("GCSFS_DEFAULT_PROJECT", "")
 
 GCS_MIN_BLOCK_SIZE = 2**18
 GCS_MAX_BLOCK_SIZE = 2**28
@@ -105,7 +106,7 @@ def _location():
     -------
     valid http location
     """
-    _emulator_location = os.environ.get("STORAGE_EMULATOR_HOST", None)
+    _emulator_location = os.getenv("STORAGE_EMULATOR_HOST", None)
     return (
         _emulator_location if _emulator_location else "https://storage.googleapis.com"
     )
@@ -303,12 +304,6 @@ class GCSFileSystem(AsyncFileSystem):
 
         self.credentials = GoogleCredentials(project, access, token)
 
-        if not self.asynchronous:
-            self._session = sync(
-                self.loop, get_client, timeout=self.timeout, **self.session_kwargs
-            )
-            weakref.finalize(self, self.close_session, self.loop, self._session)
-
     @property
     def _location(self):
         return self._endpoint or _location()
@@ -335,6 +330,7 @@ class GCSFileSystem(AsyncFileSystem):
     async def _set_session(self):
         if self._session is None:
             self._session = await get_client(**self.session_kwargs)
+            weakref.finalize(self, self.close_session, self.loop, self._session)
         return self._session
 
     @property
@@ -741,6 +737,19 @@ class GCSFileSystem(AsyncFileSystem):
 
     rmdir = sync_wrapper(_rmdir)
 
+    def modified(self, path):
+        return self._parse_timestamp(self.info(path)["updated"])
+
+    def created(self, path):
+        return self._parse_timestamp(self.info(path)["timeCreated"])
+
+    def _parse_timestamp(self, timestamp):
+        assert timestamp.endswith("Z")
+        UTC = timezone(timedelta(0))
+        timestamp = timestamp[:-1]
+        timestamp = timestamp + "0" * (6 - len(timestamp.rsplit(".", 1)[1]))
+        return datetime.fromisoformat(timestamp).astimezone(UTC)
+
     async def _info(self, path, generation=None, **kwargs):
         """File information about this path."""
         path = self._strip_protocol(path)
@@ -815,14 +824,26 @@ class GCSFileSystem(AsyncFileSystem):
             prefix = path[:ind].split("/")[-1]
         return await super()._glob(path, prefix=prefix, **kwargs)
 
-    async def _ls(self, path, detail=False, prefix="", **kwargs):
+    async def _ls(self, path, detail=False, prefix="", versions=False, **kwargs):
         """List objects under the given '/{bucket}/{prefix} path."""
         path = self._strip_protocol(path).rstrip("/")
 
         if path in ["/", ""]:
             out = await self._list_buckets()
         else:
-            out = await self._list_objects(path, prefix=prefix)
+            out = []
+            for entry in await self._list_objects(
+                path, prefix=prefix, versions=versions
+            ):
+                if versions:
+                    out.append(
+                        {
+                            **entry,
+                            "name": f"{entry['name']}#{entry['generation']}",
+                        }
+                    )
+                else:
+                    out.append(entry)
 
         if detail:
             return out
@@ -838,7 +859,7 @@ class GCSFileSystem(AsyncFileSystem):
             self._location,
             bucket,
             object,
-            "&generation={}".format(generation) if generation else "",
+            f"&generation={generation}" if generation else "",
         )
 
     async def _cat_file(self, path, start=None, end=None, **kwargs):
@@ -1164,7 +1185,7 @@ class GCSFileSystem(AsyncFileSystem):
     async def _isdir(self, path):
         try:
             return (await self._info(path))["type"] == "directory"
-        except IOError:
+        except OSError:
             return False
 
     async def _find(
@@ -1229,7 +1250,7 @@ class GCSFileSystem(AsyncFileSystem):
         self, rpath, lpath, *args, headers=None, callback=None, **kwargs
     ):
         consistency = kwargs.pop("consistency", self.consistency)
-
+        await self._set_session()
         async with self.session.get(
             url=rpath,
             params=self._get_params(kwargs),
@@ -1370,13 +1391,15 @@ class GCSFileSystem(AsyncFileSystem):
         """
         from google.cloud import storage
 
-        bucket, key = self.split_path(path)
+        bucket, key, generation = self.split_path(path)
         client = storage.Client(
             credentials=self.credentials.credentials, project=self.project
         )
         bucket = client.bucket(bucket)
         blob = bucket.blob(key)
-        return blob.generate_signed_url(expiration=expiration, **kwargs)
+        return blob.generate_signed_url(
+            expiration=expiration, generation=generation, **kwargs
+        )
 
 
 GoogleCredentials.load_tokens()
@@ -1719,11 +1742,7 @@ async def simple_upload(
     template = (
         "--==0=="
         "\nContent-Type: application/json; charset=UTF-8"
-        "\n\n"
-        + metadata
-        + "\n--==0=="
-        + "\nContent-Type: {0}".format(content_type)
-        + "\n\n"
+        "\n\n" + metadata + "\n--==0==" + f"\nContent-Type: {content_type}" + "\n\n"
     )
 
     data = template.encode() + datain + b"\n--==0==--"
